package com.rootsoft.bitmaplibrary;

import java.io.OutputStream;
import java.nio.Buffer;

import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.graphics.BitmapShader;
import android.graphics.Matrix;
import android.graphics.Rect;
import anywheresoftware.b4a.BA;
import anywheresoftware.b4a.BA.Author;
import anywheresoftware.b4a.BA.Permissions;
import anywheresoftware.b4a.BA.ShortName;
import anywheresoftware.b4a.BA.Version;


@ShortName("BitmapExtended")
@Permissions(values={"android.permission.BIND_WALLPAPER", "android.permission.SET_WALLPAPER"})
@Author("XverhelstX")
@Version(1.0f)
public class BitmapLibrary {
	
	private BA ba; 
	public String JPEG;
	public String PNG;
	public String ALPHA_8;
	public String ARGB_4444;
	public String ARGB_8888;
	public String RGB_565;
	
	
	/**
     * Initializes BitmapExtended Library.
     *
     */
	
	public void Initialize(BA ba, String Eventname)
	{
	  this.ba = ba;
	  JPEG = "JPEG";
	  PNG = "PNG";
	  ALPHA_8 = " ALPHA_8";
	  ARGB_4444 = "ARGB_4444";
	  ARGB_8888 = "ARGB_8888";
	  RGB_565 = "RGB_565";
	
	  
	}
	/**
	 * Write a compressed version of the bitmap to the specified outputstream.
	 * 
	 * format	The format of the compressed image
	 * quality	Hint to the compressor, 0-100. 0 meaning compress for small size, 100 meaning compress for max quality. Some formats, like PNG which is lossless, will ignore the quality setting
	 * stream	The outputstream to write the compressed data.
	 */
	public void compress (Bitmap bitmap, String format, int quality, OutputStream stream) {
		if (format.equals(JPEG)) {
		bitmap.compress(Bitmap.CompressFormat.JPEG, quality, stream);
		}
		
		if (format.equals(PNG)) {
			bitmap.compress(Bitmap.CompressFormat.PNG, quality, stream);
		}
	}
	
	/**
	 * Creates a new bitmap, scaled from an existing bitmap.
	 * src	The source bitmap.
	 * dstWidth	The new bitmap's desired width.
	 * dstHeight	The new bitmap's desired height.
	 * filter	true if the source should be filtered.

	 * @return the bitmap
	 */
	public Bitmap createScaledBitmap (Bitmap src, int dstWidth, int dstHeight, boolean filter) {
		Bitmap btmp = Bitmap.createScaledBitmap(src, dstWidth, dstHeight, filter);
		
		return btmp;
		
		
	}

	/**
	 * Fills the bitmap's pixels with the specified Color.
	 * @param bitmap the source bitmap. (must be mutable!)
	 * @param c colours
	 */
	public void eraseColor (Bitmap bitmap, int c) throws IllegalStateException {
		bitmap.eraseColor(c);
		
	}
	
	/**
	 * Returns a new bitmap that captures the alpha values of the original.
	 * 
	 * @param bitmap
	 * @returnnew bitmap containing the alpha channel of the original bitmap.
	 */
	public Bitmap extractAlpha (Bitmap bitmap) {
		Bitmap btmp = bitmap.extractAlpha();
		return btmp;
	}
	
	/**
	 * If the bitmap's internal config is in one of the public formats, return that config, otherwise return null.
	 * @param bitmap
	 * @return
	 */
	public Bitmap.Config getConfig (Bitmap bitmap) {	
		return bitmap.getConfig();
		
	}
	
	/**
	 * Returns the density for this bitmap.
	 * @param bitmap
	 * @return
	 */
	public int getDensity (Bitmap bitmap) {
		return bitmap.getDensity();
	}
	
	/**
	 * Returns an optional array of private data, used by the UI system for some bitmaps. Not intended to be called by applications.
	 * @param bitmap
	 * @return
	 */
	public byte[] getNinePatchChunk (Bitmap bitmap) {
		return bitmap.getNinePatchChunk();
		
	}
	
	/**
	 * Returns the bitmap's height
	 * @param bitmap
	 * @return
	 */
	public final int getHeight (Bitmap bitmap) {
		return bitmap.getHeight();
	}
	
	/**
	 * Returns the Color at the specified location. 
	 * Throws an exception if x or y are out of bounds (negative or >= to the width or height respectively).
	 * @param bitmap
	 * @param x The x coordinate (0...width-1) of the pixel to return
	 * @param y The y coordinate (0...height-1) of the pixel to return
	 * @return The argb Color at the specified coordinate
	 */
	public int getPixel (Bitmap bitmap, int x, int y) throws IllegalArgumentException{
		return bitmap.getPixel(x, y);
		
	}
	
	/**
	 * Returns in pixels[] a copy of the data in the bitmap. Each value is a packed int representing a Color. 
	 * The stride parameter allows the caller to allow for gaps in the returned pixels array between rows. 
	 * For normal packed results, just pass width for the stride value
	 * 
	 * @param bitmap
	 * @param pixels The array to receive the bitmap's colors
	 * @param offset The first index to write into pixels[]
	 * @param stride The number of entries in pixels[] to skip between rows (must be >= bitmap's width). Can be negative.
	 * @param x The x coordinate of the first pixel to read from the bitmap
	 * @param y The y coordinate of the first pixel to read from the bitmap
	 * @param The number of pixels to read from each row
	 * @param height The number of rows to read
	 * @throws IllegalArgumentException if x, y, width, height exceed the bounds of the bitmap, or if abs(stride) < width.
	 * @throws ArrayIndexOutOfBoundsException if the pixels array is too small to receive the specified number of pixels.
	 */
	public void getPixels (Bitmap bitmap, int[] pixels, int offset, int stride, int x, int y, int width, int height) throws IllegalArgumentException, ArrayIndexOutOfBoundsException {
		bitmap.getPixels(pixels, offset, stride, x, y, width, height);
	}
	
	/**
	 * Return the number of bytes between rows in the bitmap's pixels. 
	 * Note that this refers to the pixels as stored natively by the bitmap. 
	 * If you call getPixels() or setPixels(), then the pixels are uniformly treated as 32bit values, 
	 * packed according to the Color class.
	 * @param bitmap
	 * @return number of bytes between rows of the native bitmap pixels.
	 */
	public int getRowBytes (Bitmap bitmap) {
		return bitmap.getRowBytes();
		
	}
	
	/**
	 * Convenience method that returns the height of this bitmap divided by the density scale factor.
	 * 
	 * @param bitmap
	 * @param targetDensity The density of the target canvas of the bitmap.
	 * @return The scaled height of this bitmap, according to the density scale factor.
	 */
	public int getScaledHeight (Bitmap bitmap, int targetDensity) {
		return bitmap.getScaledHeight(targetDensity);
		
	}
	
	/**
	 * Convenience method that returns the width of this bitmap divided by the density scale factor.
	 * 
	 * @param bitmap
	 * @param targetDensity The density of the target canvas of the bitmap.
	 * @return The scaled width of this bitmap, according to the density scale factor.
	 */
	public int getScaledWidth (Bitmap bitmap, int targetDensity) {
		return bitmap.getScaledWidth(targetDensity);
	
	}
	
	/**
	 * Returns the bitmap's width
	 * @param bitmap
	 * @return
	 */
	public int getWidth (Bitmap bitmap) {
		return bitmap.getWidth();		
	}
	
	/**
	 * Returns true if the bitmap's config supports per-pixel alpha, and if the pixels may contain non-opaque alpha values. 
	 * For some configs, this is always false (e.g. RGB_565), since they do not support per-pixel alpha. 
	 * However, for configs that do, the bitmap may be flagged to be known that all of its pixels are opaque. 
	 * In this case hasAlpha() will also return false. If a config such as ARGB_8888 is not so flagged, it will return true by default.
	 * @param bitmap
	 * @return
	 */
	public boolean hasAlpha (Bitmap bitmap) {
		return bitmap.hasAlpha();
		
	}
	
	/**
	 * Returns true if the bitmap is marked as mutable (i.e. can be drawn into)
	 * @param bitmap
	 * @return
	 */
	public boolean isMutable (Bitmap bitmap){
		return bitmap.isMutable();
		
	}
	
	/**
	 * Returns true if this bitmap has been recycled. If so, then it is an error to try to access its pixels, and the bitmap will not draw.
	 * @param bitmap
	 * @return true if the bitmap has been recycled
	 */
	public final boolean isRecycled (Bitmap bitmap) {
		return bitmap.isRecycled();
		
	}
	
	/**
	 * Rebuilds any caches associated with the bitmap that are used for drawing it. 
	 * In the case of purgeable bitmaps, this call will attempt to ensure that the pixels have been decoded. 
	 * If this is called on more than one bitmap in sequence, the priority is given in LRU order 
	 * (i.e. the last bitmap called will be given highest priority). For bitmaps with no associated caches, 
	 * this call is effectively a no-op, and therefore is harmless.
	 * @param bitmap
	 */
	public void prepareToDraw (Bitmap bitmap) {
		bitmap.prepareToDraw();
		
	}
	
	/**
	 * Free the native object associated with this bitmap, and clear the reference to the pixel data. 
	 * This will not free the pixel data synchronously; it simply allows it to be garbage collected if 
	 * there are no other references. The bitmap is marked as "dead", meaning it will throw an exception if 
	 * getPixels() or setPixels() is called, and will draw nothing. This operation cannot be reversed, 
	 * so it should only be called if you are sure there are no further uses for the bitmap. 
	 * This is an advanced call, and normally need not be called, since the normal GC process will free 
	 * up this memory when there are no more references to this bitmap.
	 * @param bitmap
	 */
	public void recycle (Bitmap bitmap) {
		bitmap.recycle();
		
	}
	
	/**
	 * Specifies the density for this bitmap. When the bitmap is drawn to a Canvas that also has a density, 
	 * it will be scaled appropriately.
	 * @param bitmap
	 * @param density The density scaling factor to use with this bitmap or DENSITY_NONE if the density is unknown.

	 */
	public void setDensity (Bitmap bitmap, int density) {
		bitmap.setDensity(density);
		
	}
	
	/**
	 * Replace pixels in the bitmap with the colors in the array. Each element in the array is a packed int prepresenting a Color
	 * 
	 * @param bitmap
	 * @param x The x coordinate of the pixel to replace (0...width-1)
	 * @param y The y coordinate of the pixel to replace (0...height-1)
	 * @param color The Color to write into the bitmap
	 */
	public void setPixel (Bitmap bitmap, int x, int y, int color) throws IllegalStateException, IllegalArgumentException {
		bitmap.setPixel(x, y, color);
		
	}
	
	/**
	 * Replace pixels in the bitmap with the colors in the array. Each element in the array is a packed int prepresenting a Color
	 * 
	 * @param bitmap
	 * @param pixels The colors to write to the bitmap
	 * @param offset The index of the first color to read from pixels[]
	 * @param stride The number of colors in pixels[] to skip between rows. Normally this value will be the same as the width of the bitmap, but it can be larger (or negative).
	 * @param x The x coordinate of the first pixel to write to in the bitmap.
	 * @param y The y coordinate of the first pixel to write to in the bitmap.
	 * @param width The number of colors to copy from pixels[] per row
	 * @param height The number of rows to write to the bitmap
	 * 
	 * @throws IllegalStateException if the bitmap is not mutable
	 * @throws IllegalArgumentException if x, y, width, height are outside of the bitmap's bounds.
	 * @throws ArrayIndexOutOfBoundsException if the pixels array is too small to receive the specified number of pixels.
	 */
	public void setPixels (Bitmap bitmap, int[] pixels, int offset, int stride, int x, int y, int width, int height) throws IllegalStateException, IllegalArgumentException, ArrayIndexOutOfBoundsException{
		bitmap.setPixels(pixels, offset, stride, x, y, width, height);
	}
	
	/**
	 * Copy the pixels from the buffer, beginning at the current position, overwriting the bitmap's pixels. 
	 * The data in the buffer is not changed in any way (unlike setPixels(), which converts from unpremultipled 32bit to 
	 * whatever the bitmap's native format is.
	 * 
	 * @param bitmap
	 * @param src
	 */
	public void copyPixelsFromBuffer (Bitmap bitmap, Buffer src) {
		bitmap.copyPixelsFromBuffer(src);
		
	}
	
	/**
	 * Copy the bitmap's pixels into the specified buffer (allocated by the caller). 
	 * An exception is thrown if the buffer is not large enough to hold all of the pixels 
	 * (taking into account the number of bytes per pixel) or if the Buffer subclass is not one of the support types 
	 * (ByteBuffer, ShortBuffer, IntBuffer).
	 * @param bitmap
	 * @param dst
	 */
	public void copyPixelsToBuffer (Bitmap bitmap, Buffer dst) {
		bitmap.copyPixelsToBuffer(dst);
		
	}
	
	/**
	 * Rotates the bitmap.
	 * @param bitmap
	 * @param degrees
	 * @return
	 */
	public Bitmap rotateBitmap(Bitmap bitmap, float degrees) {
		Matrix matrix = new Matrix();
    	matrix.postRotate(degrees);
    	Bitmap newBitmap = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);
    	return newBitmap;
	}
	
	/**
	 * Scales the bitmap.
	 * @param bitmap
	 * @param degrees
	 * @return
	 */
	public Bitmap scaleBitmap(Bitmap bitmap, float sx, float sy) {
		Matrix matrix = new Matrix();
    	matrix.postScale(sx, sy);
    	Bitmap newBitmap = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);
    	return newBitmap;
	}
	
	/**
	 * Skews the bitmap.
	 * @param bitmap
	 * @param kx
	 * @param ky
	 * @return
	 */
	public Bitmap skewBitmap(Bitmap bitmap, float kx, float ky) {
		Matrix matrix = new Matrix();
    	matrix.postSkew(kx, ky);
    	Bitmap newBitmap = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);
    	return newBitmap;
	}
	
	/**
	 * Skews the bitmap.
	 * @param bitmap
	 * @param degrees
	 * @return
	 */
	public Bitmap translateBitmap(Bitmap bitmap, float dx, float dy) {
		Matrix matrix = new Matrix();
    	matrix.postTranslate(dx, dy);
    	Bitmap newBitmap = Bitmap.createBitmap(bitmap, 0, 0, bitmap.getWidth(), bitmap.getHeight(), matrix, true);
    	return newBitmap;
	}
	
	/**
	 * Returns an immutable bitmap from subset of the source bitmap, transformed by the optional matrix. 
	 * It is initialized with the same density as the original bitmap.
	 * @param source
	 * @param x
	 * @param y
	 * @param width
	 * @param height
	 * @param m
	 * @param filter
	 * @return
	 */
	public Bitmap createBitmap (Bitmap source, int x, int y, int width, int height, Matrix m, boolean filter) throws IllegalArgumentException {
		return Bitmap.createBitmap(source, x, y, width, height, m, filter);
	}
	
	/**
	 * Returns a mutable bitmap with the specified width and height.
	 * 
	 * @param width
	 * @param height
	 * @param config
	 * @return
	 */
	public Bitmap createBitmap2 (int width, int height, String config) throws IllegalArgumentException {
		if (config.equals(ALPHA_8)) {
			return Bitmap.createBitmap(width, height, Bitmap.Config.ALPHA_8);
			}
		if (config.equals(ARGB_4444)) {
			return Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_4444);
			}
		if (config.equals(ARGB_8888)) {
			return Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);
			}
		if (config.equals(RGB_565 )) {
			return Bitmap.createBitmap(width, height, Bitmap.Config.RGB_565);
			}
		return null;
		
	}
	
	/**
	 * Returns an immutable bitmap from the specified subset of the source bitmap. 
	 * The new bitmap may be the same object as source, or a copy may have been made. 
	 * It is initialized with the same density as the original bitmap.
	 * 
	 * @param source
	 * @param x
	 * @param y
	 * @param width
	 * @param height
	 * @return
	 */
	public Bitmap createBitmap3 (Bitmap source, int x, int y, int width, int height) {
		return Bitmap.createBitmap(source, x, y, width, height);
	}
	
	/**
	 * Returns an immutable bitmap from the source bitmap. 
	 * The new bitmap may be the same object as source, or a copy may have been made. 
	 * It is initialized with the same density as the original bitmap.
	 * @param src
	 * @return
	 */
	public Bitmap createBitmap4 (Bitmap src) {
		return Bitmap.createBitmap(src);
	}
	
	public Bitmap createBitmap5 (int[] colors, int width, int height, String config) {
		if (config.equals(ALPHA_8)) {
			return Bitmap.createBitmap(colors, width, height, Bitmap.Config.ALPHA_8);
			}
		if (config.equals(ARGB_4444)) {
			return Bitmap.createBitmap(colors, width, height, Bitmap.Config.ARGB_4444);
			}
		if (config.equals(ARGB_8888)) {
			return Bitmap.createBitmap(colors, width, height, Bitmap.Config.ARGB_8888);
			}
		if (config.equals(RGB_565 )) {
			return Bitmap.createBitmap(colors, width, height, Bitmap.Config.RGB_565);
			}
		return null;
		
	}
		
}
	

